var WebTag = {

  logged_in: false,

  onLoad: function() {
    var appcontent = document.getElementById("appcontent");
    if(appcontent) {
      appcontent.addEventListener("load", WebTag.onPageLoad, true);
    }

    // localization strings
    var strings = document.getElementById("webtag_bundle");

    WebTag.s_please_login =             strings.getString('webtag_please_login');
    WebTag.s_found_a_tag =              strings.getString('webtag_found_a_tag');
    WebTag.s_login_server_unavailable = strings.getString('webtag_login_server_unavailable');
    WebTag.s_login_error =              strings.getString('webtag_login_error');
    WebTag.s_login_more_info =          strings.getString('webtag_login_more_info');
    WebTag.s_now_logged_in =            strings.getString('webtag_now_logged_in');
//    WebTag.s_ = strings.getString('webtag_');
//    alert("onload: autologin");
    WebTag.autoLogin();

    if (document.getElementById("webtag_enabled")) {
      // only checking when it can update the menu item
//      alert("onload: checklogin");
      WebTag.checkLogin(function(){}, false);
    }
  },

  onPageLoad: function(aEvent) {
    if (WebTag.isEnabled() &&                            // only check if we are enabled
	aEvent.originalTarget.nodeName == "#document") { // and we have an actual page to check
      WebTag.checkPage(aEvent.originalTarget.location.href);
    }
  },

  isEnabled: function() {
    return document.getElementById("webtag_enabled").getAttribute("checked");
  },

  onToolbarButtonClick: function(event) {
    switch(event.button) {
      case 0:
        // Left click
        // Try to place a tag on the currently view page
        WebTag.tagPage(WebTag.getURL());
        break;
      case 1:
        // Middle click, nothing for now
        break;
      case 2:
        // Right click, nothing for now
        break;
    }
  },

  tagPage: function(href) {
    if (WebTag.logged_in) {
      var func = function(event) {
	WebTag.sendMessage(event.target.responseText);
      }
      WebTag.makePOSTReq('', 'f=place_tag&href=' + escape(href), func);
    } else {
      WebTag.checkLogin(function(){WebTag.tagPage(href);}, true);
    }
  },

  checkLogin: function(doIfLoggedIn, promptForLogin) {
      var lfunc = function(event) {
	if (event.target.responseText == '1') {
	  WebTag.logged_in = true;
	  document.getElementById("webtag_enabled").setAttribute("checked", true);
	  doIfLoggedIn();
	} else {
	  if (WebTag.autoLogin()) {
//	    alert("auto login success");
	    doIfLoggedIn();
	  } else {
//	    alert("checklogin: promptforloginvalue: "+promptForLogin);
	    if (promptForLogin) {
	      WebTag.sendMessage(WebTag.s_please_login);
	    }
	  }
	}
      }
      WebTag.makeGETReq('?is_logged_in', lfunc);
  },

  getURL: function() {
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].
    getService(Components.interfaces.nsIWindowMediator);
    var recentWindow = wm.getMostRecentWindow("navigator:browser");
    return recentWindow ? recentWindow.content.document.location : null;
  },

  checkPage: function(href) {
    var func = function(event) {
      var parts = event.target.responseText.split("|");
//      WebTag.sendMessage(parts[0]);
      if (parts[0] == "1") {
	// the've found a tag
	WebTag.sendMessage(WebTag.s_found_a_tag);
      } else if (parts[0] == "Only http pages are allowed.") {
	// This message can be ignored on checks
	return;
      } else if (parts[0] != "0") {
	// Some other message TODO: make this message localizable via a *.properties file
//	WebTag.sendMessage(parts[0]);
//	alert(parts[0]);
      }
    }

    if (href.substr(0, 7) == "http://") {
      WebTag.makeGETReq('?check&href=' + escape(href), func);
    }
  },

  sendMessage: function(message) {
    window.openDialog("chrome://webtag/content/message.xul", "WebTagMessage", "chrome,dialog=yes,titlebar=no,popup=yes", message);
  },

  getPrefService: function() {
    return Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("extensions.webtag.");
  },

  encryptPW: function(pass) {
    return escape(TEAencrypt(pass, document.getElementById("webtag_bundle").getString("webtag_password_encrypting_password")));
  },

  decryptPW: function(cyphertext) {
    return TEAdecrypt(unescape(cyphertext), document.getElementById("webtag_bundle").getString("webtag_password_encrypting_password"));
  },

  makeGETReq: function(args, func) {
    var req = new XMLHttpRequest();
    req.open('GET', 'http://mmatton.dyndns.org/~webtag/ext.php' + args, true);
    req.onload = func;
    req.send(null);
  },

  makePOSTReq: function(getargs, data, func) {
    var req = new XMLHttpRequest();
    req.open('POST', 'http://mmatton.dyndns.org/~webtag/ext.php' + getargs, true);
    req.onload = func;
    req.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
    req.send(data);
  },

  toggleOn: function() {
    if (!WebTag.logged_in && WebTag.isEnabled()) {
      var w = window.openDialog("chrome://webtag/content/login.xul", "WebTagLogin", "chrome,dialog=yes", document.getElementById('webtag_enabled'));
    }
  },

  // This is called by the accept button of the login window
  onLoginAccept: function() {
    var loginValid = WebTag.login(document.getElementById("webtag_name").value, document.getElementById("webtag_pass").value);
    if (loginValid) {
      WebTag.logged_in = true;
      var checkbox = document.getElementById("webtag_auto_login");
      if (checkbox.checked) {
	var prefs = WebTag.getPrefService();
	prefs.setBoolPref("auto_login", true);
	prefs.setCharPref("username", escape(document.getElementById("webtag_name").value));
	prefs.setCharPref("epassword", WebTag.encryptPW(document.getElementById("webtag_pass").value));
      }
      document.getElementById("webtag_pass").value = ""; // blank this out since I'm not sure whether it survives or not
    }
    return loginValid;
  },

  autoLogin: function() {
    try {
      // see if they have auto-login enabled
      var prefs = WebTag.getPrefService();
      if (prefs.prefHasUserValue("auto_login") && prefs.getBoolPref("auto_login")) {
	if (prefs.prefHasUserValue("username") && prefs.prefHasUserValue("epassword")) {
          if (WebTag.login(unescape(prefs.getCharPref("username")), WebTag.decryptPW(prefs.getCharPref("epassword")))) {
	    document.getElementById("webtag_enabled").setAttribute("checked", true);
	    WebTag.logged_in = true;
	    return true;
	  } else {
	    prefs.setBoolPref("auto_login", false);
	  }
	} else {
	  prefs.setBoolPref("auto_login", false);
	}
      }
    } catch (ex) {}
    return false;
  },

  login: function(name, pass) {
    if (name == "" || pass == "") {
      if (document.getElementById("webtag_login_feedback")) {
	document.getElementById("webtag_login_feedback").value = WebTag.s_login_more_info;
      } else {
	WebTag.message(WebTag.s_login_more_info);
      }
      return false;
    }

    var req = new XMLHttpRequest();
    req.open('POST', 'http://mmatton.dyndns.org/~webtag/ext.php', false);
    req.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
    req.send("login=login&n=" + escape(name) + "&p=" + escape(pass));

    if (req.status == 200) {
      if (req.responseText == "1") {
	WebTag.logged_in = true;
	WebTag.sendMessage(WebTag.s_now_logged_in);
	return true;
      } else {
	if (req.responseText) {
	  WebTag.sendMessage(req.responseText);
	} else {
	  WebTag.sendMessage(WebTag.s_login_error);
	}
	if (document.getElementById("webtag_login_feedback")) {
	  document.getElementById("webtag_login_feedback").value = WebTag.s_login_error;
	}
	return false;
      }
    } else {
      if (document.getElementById("webtag_login_feedback")) {
	document.getElementById("webtag_login_feedback").value = WebTag.s_login_server_unavailable;
      } else {
	WebTag.message(WebTag.s_login_server_unavailable);
      }
      return false;
    }

    return false;
  }
};

window.addEventListener("load", function(e) { WebTag.onLoad(e); }, false);
